// ------------------------------------------------------------------
//	Class	TIcon
//	Base	TView
//
// This class is a simple icon linked to a TMinWindow. If the icon
//	has the focus and you 1) double-click the mouse on it, or
//	2) press the ENTER key, it calls the virtual method zoom().
//	The icon does NOT support "growing", but it DOES support "moving".
// ------------------------------------------------------------------

#define Uses_TIcon
#define Uses_TMinWindow
#define Uses_TEvent
#define Uses_TKeys
#define Uses_TPalette
#define Uses_TDrawBuffer
#define Uses_TCommandSet
#define Uses_opstream
#define Uses_ipstream
#define Uses_TStreamable
#define Uses_TStreamableClass
#include "main.h"

__link( RView )
const char * const near TIcon::name = "TIcon";
TStreamableClass RIcon( TIcon::name, TIcon::build, __DELTA(TIcon) );

// ------------------------------------------------------------------
// Create an icon that is not linked to a window.
// ------------------------------------------------------------------

TIcon::TIcon( const TRect& bounds ) : TView( bounds ), link(0)
	{
	options |= ofSelectable | ofTopSelect;
	dragMode |= dmDragMove;
	eventMask |= evCommand;
	}

// ------------------------------------------------------------------
// When an icon is told to shut down, it can't forget its linked
//	TMinWindow. The default process, if a window is linked, is to:
//		1) NULL the window's icon member, so we don't get into a
//			vicious cycle.
//		2) Remove the window from its TGroup owner, if it has one.
//		3) destroy the window.
//		4) destroy the icon.
// ------------------------------------------------------------------

void TIcon::shutDown()
	{
	if( linkIsValid() )
		{
		link->icon = 0;
		if( link->owner == 0 )
			destroy(link);
		}
	link = 0;
	TView::shutDown();
	}

// Respond to cmResize (by moving, NOT growing), cmClose by calling the
//	virtual method close(), cmZoom by calling the virtual method zoom(),
//	doubleClicks by calling zoom(), clicks by resizing, and the
//	ENTER key by calling zoom().

void TIcon::handleEvent( TEvent& event )
	{
	TView::handleEvent(event);

	if( event.what== evCommand )
		switch (event.message.command)
			{
			case  cmResize:
				TRect limits = owner->getExtent();
				dragView( event, dmDragMove, limits, size, size);
				clearEvent(event);
				break;
			case  cmClose:
				if( event.message.infoPtr == 0 ||
				  event.message.infoPtr == this )
					{
					close();
					clearEvent( event );
					}
				break;
			case  cmZoom:
				if( event.message.infoPtr == 0 ||
				  event.message.infoPtr == this )
					{
					zoom();
					clearEvent(event);
					}
				break;
			}

	else if( (state & sfSelected) != 0 )
		{
		if( event.what== evMouseDown )
			{
			if( event.mouse.doubleClick )
				{
				zoom();
				clearEvent(event);
				}
			else
				{
				TRect rLimits = owner->getExtent();
				dragView( event, dmDragMove, rLimits, size, size );
				clearEvent(event);
				}
			}
		else if( event.what == evKeyDown && event.keyDown.keyCode == kbEnter )
			{
			zoom();
			clearEvent(event);
			}
		}
	}

// Similar to TWindow::setState(); enables window-type commands.
// 	Also, like TFrame, redraws if the window sfSelected has
//	changed or are currently dragging.

void TIcon::setState( ushort aState, Boolean enable )
	{
	TCommandSet windowCommands;

	TView::setState( aState, enable );
	if( (aState & sfSelected) != 0 )
		{
		setState(sfActive, enable);
		windowCommands += cmNext;
		windowCommands += cmPrev;
		windowCommands += cmResize;
		if( linkIsValid() && (link->flags & wfClose) != 0 )
			windowCommands += cmClose;
		windowCommands += cmZoom;
		if( enable != False )
			enableCommands(windowCommands);
		else
			disableCommands(windowCommands);
		}

	if( (aState & (sfSelected | sfDragging)) != 0 )
		drawView();
	}

// Use the linked window's palette (if linked).

TPalette& TIcon::getPalette() const
	{
	if( link != 0 && link->icon == this )
		return link->getPalette();
	else
		return TView::getPalette();
	}

void TIcon::draw()
	{
	TDrawBuffer b;
	ushort color;

	if( (state & sfSelected) == 0 )
		color = 0x0101;
	else
		if( (state & sfDragging) != 0 )
			color = 0x0303;
		else
			color = 0x0302;
	color = getColor(color);

	b.moveChar( 0, ' ', color, size.x );
	writeLine( 0, 0, size.x, size.y, b );
	}

// When valid() is called with cmClose, check the linked window
//	for valid(cmClose), so that cleanup can be performed.

Boolean TIcon::valid( ushort command )
	{
	if( linkIsValid() && command == cmClose &&
	  link->valid(command) == False )
		return False;
	return TView::valid(command);
	}

// Destroy the icon/window if they can close.

void TIcon::close()
	{
	if( valid( cmClose ) )
		destroy( this );
	}

// Restore the window by removing the icon and show()ing the window.

void TIcon::zoom()
	{
	if( linkIsValid() )
		{
		TGroup *o = owner;
		o->lock();
		o->insertBefore( link, this );
		o->remove( this );
		o->unlock();
		}
	}

Boolean TIcon::linkIsValid()
	{
	return Boolean( link != 0 && link->icon == this );
	}

TIcon::TIcon( StreamableInit ) : TView( streamableInit )
	{
	}

void TIcon::write( opstream& os )
	{
	TView::write(os);
	os << link;
	}

void *TIcon::read( ipstream& is )
	{
	TView::read(is);
	is >> link;
	return this;
	}

TStreamable *TIcon::build()
	{
	return new TIcon( streamableInit );
	}
